from .insn import *

"""
RV32I SB-Type Instruction
"""

class InsnRV32ISBType(Insn):
    def __init__(self, params, funct3):
        super().__init__(params)
        self.funct3 = funct3

    def elaborate(self, platform):
        m = super().elaborate(platform)

        m.d.comb += self.insn_imm.eq(Value.as_signed(Cat(self.rvfi_insn[8:12], self.rvfi_insn[25:31], self.rvfi_insn[7], self.rvfi_insn[31]) << 1))

        if self.params.csr_misa:
            m.d.comb += self.misa_ok.eq((self.rvfi_csr_misa_rdata & 0) == 0)
            m.d.comb += self.spec_csr_misa_rmask.eq(4)
            m.d.comb += self.ialign16.eq((self.rvfi_csr_misa_rdata & 4) != 0)
        else:
            m.d.comb += self.misa_ok.eq(1)
            if self.params.compressed:
                m.d.comb += self.ialign16.eq(1)
            else:
                m.d.comb += self.ialign16.eq(0)

        m.d.comb += self.spec_valid.eq(self.rvfi_valid & (~self.insn_padding) & (self.insn_funct3 == self.funct3) & (self.insn_opcode == 0b1100011))
        m.d.comb += self.spec_rs1_addr.eq(self.insn_rs1)
        m.d.comb += self.spec_rs2_addr.eq(self.insn_rs2)

        return m
